@@ -28,6 +28,7 @@ import Globals, Time, SetConnections, selection, robust, rpath, \
 
 
 action = None
+create_full_path = None
 remote_cmd, remote_schema = None, None
 force = None
 select_opts = []
@@ -40,8 +41,8 @@ return_val = None # Set to cause exit code to be specified value
 
 def parse_cmdlineoptions(arglist):
 	"""Parse argument list and set global preferences"""
-	global args, action, force, restore_timestr, remote_cmd, remote_schema
-	global remove_older_than_string
+	global args, action, create_full_path, force, restore_timestr, remote_cmd
+	global remote_schema, remove_older_than_string
 	global user_mapping_filename, group_mapping_filename
 	def sel_fl(filename):
 		"""Helper function for including/excluding filelists below"""
@@ -54,11 +55,11 @@ def parse_cmdlineoptions(arglist):
 
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
 		 ["backup-mode", "calculate-average", "check-destination-dir",
-		  "compare", "compare-at-time=", "current-time=", "exclude=",
-		  "exclude-device-files", "exclude-fifos",
-		  "exclude-filelist=", "exclude-symbolic-links",
-		  "exclude-sockets", "exclude-filelist-stdin",
-		  "exclude-globbing-filelist=",
+		  "compare", "compare-at-time=", "create-full-path",
+		  "current-time=", "exclude=", "exclude-device-files",
+		  "exclude-fifos", "exclude-filelist=",
+		  "exclude-symbolic-links", "exclude-sockets",
+		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
 		  "exclude-globbing-filelist-stdin", "exclude-mirror=",
 		  "exclude-other-filesystems", "exclude-regexp=",
 		  "exclude-special-files", "force", "group-mapping-file=",
@@ -88,6 +89,7 @@ def parse_cmdlineoptions(arglist):
 			action = "compare"
 			if opt == "--compare": restore_timestr = "now"
 			else: restore_timestr = arg
+		elif opt == "--create-full-path": create_full_path = 1
 		elif opt == "--current-time":
 			Globals.set_integer('current_time', arg)
 		elif (opt == "--exclude" or
@@ -295,8 +297,8 @@ def Backup(rpin, rpout):
 	backup_set_select(rpin)
 	backup_warn_if_infinite_regress(rpin, rpout)
 	if prevtime:
-		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 		Time.setprevtime(prevtime)
+		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 		backup.Mirror_and_increment(rpin, rpout, incdir)
 		rpout.conn.Main.backup_remove_curmirror_local()
 	else:
@@ -325,7 +327,9 @@ def backup_check_dirs(rpin, rpout):
 			Log("Deleting %s" % rpout.path, 3)
 			rpout.delete()
 	if not rpout.lstat():
-		try: rpout.mkdir()
+		try: 
+			if create_full_path: rpout.makedirs()
+			else: rpout.mkdir()
 		except os.error:
 			Log.FatalError("Unable to create directory %s" % rpout.path)
 
@@ -407,10 +411,10 @@ def backup_set_fs_globals(rpin, rpout):
 			rpout.conn.Globals.set_local(conn_attr, 1)
 
 	src_fsa = rpin.conn.fs_abilities.get_fsabilities_readonly('source', rpin)
-	Log(str(src_fsa), 3)
+	Log(str(src_fsa), 4)
 	dest_fsa = rpout.conn.fs_abilities.get_fsabilities_readwrite(
 		'destination', Globals.rbdir, 1, Globals.chars_to_quote)
-	Log(str(dest_fsa), 3)
+	Log(str(dest_fsa), 4)
 
 	update_triple(src_fsa.eas, dest_fsa.eas,
 				  ('eas_active', 'eas_write', 'eas_conn'))
@@ -514,10 +518,10 @@ def restore_set_fs_globals(target):
 
 	target_fsa = target.conn.fs_abilities.get_fsabilities_readwrite(
 		'destination', target, 0, Globals.chars_to_quote)
-	Log(str(target_fsa), 3)
+	Log(str(target_fsa), 4)
 	mirror_fsa = Globals.rbdir.conn.fs_abilities.get_fsabilities_restoresource(
 		Globals.rbdir)
-	Log(str(mirror_fsa), 3)
+	Log(str(mirror_fsa), 4)
 
 	update_triple(mirror_fsa.eas, target_fsa.eas,
 				  ('eas_active', 'eas_write', 'eas_conn'))
@@ -701,7 +705,7 @@ def single_set_fs_globals(rbdir):
 
 	fsa = rbdir.conn.fs_abilities.get_fsabilities_readwrite('archive',
 								   rbdir, 1, Globals.chars_to_quote)
-	Log(str(fsa), 3)
+	Log(str(fsa), 4)
 
 	update_triple(fsa.eas, ('eas_active', 'eas_write', 'eas_conn'))
 	update_triple(fsa.acls, ('acls_active', 'acls_write', 'acls_conn'))
