@@ -78,7 +78,8 @@ def set_security_level(action, cmdpairs):
 			sec_level = "read-only"
 			Main.restore_set_root(rpath.RPath(Globals.local_connection,
 											  getpath(cp1)))
-			rdir = Main.restore_root.path
+			if Main.restore_root: rdir = Main.restore_root.path
+			else: log.Log.FatalError("Invalid restore directory")
 		else:
 			assert islocal(cp2)
 			sec_level = "all"
@@ -195,6 +196,7 @@ def vet_rpath(rpath):
 	"""Require rpath not to step outside retricted directory"""
 	if Globals.restrict_path and rpath.conn is Globals.local_connection:
 		normalized, restrict = rpath.normalize().path, Globals.restrict_path
+		if restrict == "/": return
 		components = normalized.split("/")
 		# 3 cases for restricted dir /usr/foo:  /var, /usr/foobar, /usr/foo/..
 		if (not normalized.startswith(restrict) or
